@@ -1,7 +1,7 @@
 /*
- * buflist.c - Bar with list of buffers
+ * SPDX-FileCopyrightText: 2003-2025 Sébastien Helleu <flashcode@flashtux.org>
  *
- * Copyright (C) 2003-2025 Sébastien Helleu <flashcode@flashtux.org>
+ * SPDX-License-Identifier: GPL-3.0-or-later
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -19,6 +19,8 @@
  * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
+/* Bar with list of buffers */
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -399,14 +401,14 @@ buflist_script_loaded_cb (const void *pointer, void *data, const char *signal,
     if (strcmp (base_name2, "buffers.pl") == 0)
     {
         weechat_printf (NULL,
-                        _("%sbuflist: warning: the script buffers.pl is "
+                        _("%s%s: warning: the script buffers.pl is "
                           "loaded and provides a bar with list of buffers "
                           "similar to the buflist plugin; you may want to "
                           "uninstall the script buffers.pl "
                           "(/script remove buffers.pl) or disable/unload the "
                           "buflist plugin; see WeeChat release notes for more "
                           "information"),
-                        weechat_prefix ("error"));
+                        weechat_prefix ("error"), BUFLIST_PLUGIN_NAME);
     }
 
     free (base_name2);
