@@ -1,5 +1,7 @@
 /*
- * Copyright (C) 2003-2015 Sébastien Helleu <flashcode@flashtux.org>
+ * SPDX-FileCopyrightText: 2003-2025 Sébastien Helleu <flashcode@flashtux.org>
+ *
+ * SPDX-License-Identifier: GPL-3.0-or-later
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -14,28 +16,24 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
-#ifndef WEECHAT_RELAY_NETWORK_H
-#define WEECHAT_RELAY_NETWORK_H 1
+#ifndef WEECHAT_PLUGIN_RELAY_NETWORK_H
+#define WEECHAT_PLUGIN_RELAY_NETWORK_H
 
-#ifdef HAVE_GNUTLS
 #include <gnutls/gnutls.h>
-#endif /* HAVE_GNUTLS */
 
 extern int relay_network_init_ok;
-extern int relay_network_init_ssl_cert_key_ok;
+extern int relay_network_init_tls_cert_key_ok;
 
-#ifdef HAVE_GNUTLS
 extern gnutls_certificate_credentials_t relay_gnutls_x509_cred;
 extern gnutls_priority_t *relay_gnutls_priority_cache;
 extern gnutls_dh_params_t *relay_gnutls_dh_params;
-#endif /* HAVE_GNUTLS */
 
-extern void relay_network_set_ssl_cert_key (int verbose);
-extern void relay_network_set_priority ();
-extern void relay_network_init ();
-extern void relay_network_end ();
+extern void relay_network_set_tls_cert_key (int verbose);
+extern void relay_network_set_priority (void);
+extern void relay_network_init (void);
+extern void relay_network_end (void);
 
-#endif /* WEECHAT_RELAY_NETWORK_H */
+#endif /* WEECHAT_PLUGIN_RELAY_NETWORK_H */
